@@ -1,9 +1,9 @@
 /*
- * "$Id: cups-private.h 10814 2013-01-14 22:06:21Z mike $"
+ * "$Id: cups-private.h 12441 2015-01-29 14:42:32Z msweet $"
  *
  *   Private definitions for CUPS.
  *
- *   Copyright 2007-2013 by Apple Inc.
+ *   Copyright 2007-2015 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -24,6 +24,7 @@
 
 #  include "string-private.h"
 #  include "debug-private.h"
+#  include "array-private.h"
 #  include "ipp-private.h"
 #  include "http-private.h"
 #  include "language-private.h"
@@ -86,6 +87,11 @@ typedef struct _cups_globals_s		/**** CUPS global state data ****/
   char			resolved_uri[1024];
 					/* Buffer for cupsBackendDeviceURI */
 
+  /* debug.c */
+#  ifdef DEBUG
+  int			thread_id;	/* Friendly thread ID */
+#  endif /* DEBUG */
+
   /* file.c */
   cups_file_t		*stdio_files[3];/* stdin, stdout, stderr */
 
@@ -125,7 +131,7 @@ typedef struct _cups_globals_s		/**** CUPS global state data ****/
   cups_array_t		*leg_size_lut,	/* Lookup table for legacy names */
 			*ppd_size_lut,	/* Lookup table for PPD names */
 			*pwg_size_lut;	/* Lookup table for PWG names */
-  _pwg_media_t		pwg_media;	/* PWG media data for custom size */
+  pwg_media_t		pwg_media;	/* PWG media data for custom size */
   char			pwg_name[65];	/* PWG media name for custom size */
 
   /* request.c */
@@ -145,6 +151,7 @@ typedef struct _cups_globals_s		/**** CUPS global state data ****/
   /* usersys.c */
   http_encryption_t	encryption;	/* Encryption setting */
   char			user[65],	/* User name */
+			user_agent[256],/* User-Agent string */
 			server[256],	/* Server address */
 			servername[256],/* Server hostname */
 			password[128];	/* Password for default callback */
@@ -159,9 +166,9 @@ typedef struct _cups_globals_s		/**** CUPS global state data ****/
   void			*server_cert_data;
 					/* Server certificate user data */
   int			server_version,	/* Server IPP version */
-			any_root,	/* Allow any root */
+			any_root,	/* Allow any (e.g., self-signed) root */
 			expired_certs,	/* Allow expired certs */
-			expired_root;	/* Allow expired root */
+			validate_certs;	/* Validate certificates */
 
   /* util.c */
   char			def_printer[256];
@@ -200,6 +207,7 @@ typedef struct _cups_dconstres_s	/* Constraint/resolver */
 struct _cups_dinfo_s			/* Destination capability and status
 					 * information */
 {
+  int			version;	/* IPP version */
   const char		*uri;		/* Printer URI */
   char			*resource;	/* Resource path */
   ipp_t			*attrs;		/* Printer attributes */
@@ -211,6 +219,11 @@ struct _cups_dinfo_s			/* Destination capability and status
   cups_array_t		*media_db;	/* Media database */
   _cups_media_db_t	min_size,	/* Minimum size */
 			max_size;	/* Maximum size */
+  unsigned		cached_flags;	/* Flags used for cached media */
+  cups_array_t		*cached_db;	/* Cache of media from last index/default */
+  time_t		ready_time;	/* When xxx-ready attributes were last queried */
+  ipp_t			*ready_attrs;	/* xxx-ready attributes */
+  cups_array_t		*ready_db;	/* media[-col]-ready media database */
 };
 
 
@@ -268,5 +281,5 @@ extern char		*_cupsUserDefault(char *name, size_t namesize);
 #endif /* !_CUPS_CUPS_PRIVATE_H_ */
 
 /*
- * End of "$Id: cups-private.h 10814 2013-01-14 22:06:21Z mike $".
+ * End of "$Id: cups-private.h 12441 2015-01-29 14:42:32Z msweet $".
  */
