@@ -1,42 +1,27 @@
 /*
- * "$Id: localize.c 10270 2012-02-13 17:13:21Z mike $"
+ * "$Id: localize.c 12834 2015-08-06 13:56:32Z msweet $"
  *
- *   PPD localization routines for CUPS.
+ * PPD localization routines for CUPS.
  *
- *   Copyright 2007-2012 by Apple Inc.
- *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
+ * Copyright 2007-2015 by Apple Inc.
+ * Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  *
- *   PostScript is a trademark of Adobe Systems, Inc.
+ * PostScript is a trademark of Adobe Systems, Inc.
  *
- *   This code and any derivative of it may be used and distributed
- *   freely under the terms of the GNU General Public License when
- *   used with GNU Ghostscript or its derivatives.  Use of the code
- *   (or any derivative of it) with software other than GNU
- *   GhostScript (or its derivatives) is governed by the CUPS license
- *   agreement.
+ * This code and any derivative of it may be used and distributed
+ * freely under the terms of the GNU General Public License when
+ * used with GNU Ghostscript or its derivatives.  Use of the code
+ * (or any derivative of it) with software other than GNU
+ * GhostScript (or its derivatives) is governed by the CUPS license
+ * agreement.
  *
- *   This file is subject to the Apple OS-Developed Software exception.
- *
- * Contents:
- *
- *   ppdLocalize()           - Localize the PPD file to the current locale.
- *   ppdLocalizeAttr()       - Localize an attribute.
- *   ppdLocalizeIPPReason()  - Get the localized version of a cupsIPPReason
- *                             attribute.
- *   ppdLocalizeMarkerName() - Get the localized version of a marker-names
- *                             attribute value.
- *   _ppdFreeLanguages()     - Free an array of languages from _ppdGetLanguages.
- *   _ppdGetLanguages()      - Get an array of languages from a PPD file.
- *   _ppdHashName()          - Generate a hash value for a device or profile
- *                             name.
- *   _ppdLocalizedAttr()     - Find a localized attribute.
- *   ppd_ll_CC()             - Get the current locale names.
+ * This file is subject to the Apple OS-Developed Software exception.
  */
 
 /*
@@ -51,7 +36,7 @@
  * Local functions...
  */
 
-static cups_lang_t	*ppd_ll_CC(char *ll_CC, int ll_CC_size);
+static cups_lang_t	*ppd_ll_CC(char *ll_CC, size_t ll_CC_size);
 
 
 /*
@@ -61,7 +46,7 @@ static cups_lang_t	*ppd_ll_CC(char *ll_CC, int ll_CC_size);
  * descriptions, printer presets, and custom option parameters.  Each
  * localized string uses the UTF-8 character encoding.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 int					/* O - 0 on success, -1 on error */
@@ -246,7 +231,7 @@ ppdLocalizeAttr(ppd_file_t *ppd,	/* I - PPD file */
  *
  * If no value of the requested scheme can be found, NULL is returned.
  *
- * @since CUPS 1.3/Mac OS X 10.5@
+ * @since CUPS 1.3/OS X 10.5@
  */
 
 const char *				/* O - Value or NULL if not found */
@@ -263,8 +248,8 @@ ppdLocalizeIPPReason(
 		*bufptr,		/* Pointer into buffer */
 		*bufend,		/* Pointer to end of buffer */
 		*valptr;		/* Pointer into value */
-  int		ch,			/* Hex-encoded character */
-		schemelen;		/* Length of scheme name */
+  int		ch;			/* Hex-encoded character */
+  size_t	schemelen;		/* Length of scheme name */
 
 
  /*
@@ -303,14 +288,14 @@ ppdLocalizeIPPReason(
       const char *message = NULL;	/* Localized message */
 
       if (!strncmp(reason, "media-needed", 12))
-	message = _("The paper tray needs to be filled.");
+	message = _("Load paper.");
       else if (!strncmp(reason, "media-jam", 9))
-	message = _("There is a paper jam.");
+	message = _("Paper jam.");
       else if (!strncmp(reason, "offline", 7) ||
 		       !strncmp(reason, "shutdown", 8))
 	message = _("The printer is not connected.");
       else if (!strncmp(reason, "toner-low", 9))
-	message = _("The printer is running low on toner.");
+	message = _("The printer is low on toner.");
       else if (!strncmp(reason, "toner-empty", 11))
 	message = _("The printer may be out of toner.");
       else if (!strncmp(reason, "cover-open", 10))
@@ -320,21 +305,21 @@ ppdLocalizeIPPReason(
       else if (!strncmp(reason, "door-open", 9))
 	message = _("The printer's door is open.");
       else if (!strncmp(reason, "input-tray-missing", 18))
-	message = _("The paper tray is missing.");
+	message = _("Paper tray is missing.");
       else if (!strncmp(reason, "media-low", 9))
-	message = _("The paper tray is almost empty.");
+	message = _("Paper tray is almost empty.");
       else if (!strncmp(reason, "media-empty", 11))
-	message = _("The paper tray is empty.");
+	message = _("Paper tray is empty.");
       else if (!strncmp(reason, "output-tray-missing", 19))
-	message = _("The output bin is missing.");
+	message = _("Output bin is missing.");
       else if (!strncmp(reason, "output-area-almost-full", 23))
-	message = _("The output bin is almost full.");
+	message = _("Output bin is almost full.");
       else if (!strncmp(reason, "output-area-full", 16))
-	message = _("The output bin is full.");
+	message = _("Output bin is full.");
       else if (!strncmp(reason, "marker-supply-low", 17))
-	message = _("The printer is running low on ink.");		
+	message = _("The printer is low on ink.");
       else if (!strncmp(reason, "marker-supply-empty", 19))
-	message = _("The printer is out of ink.");
+	message = _("The printer may be out of ink.");
       else if (!strncmp(reason, "marker-waste-almost-full", 24))
 	message = _("The printer's waste bin is almost full.");
       else if (!strncmp(reason, "marker-waste-full", 17))
@@ -351,7 +336,7 @@ ppdLocalizeIPPReason(
 	message = _("The developer unit will need to be replaced soon.");
       else if (!strncmp(reason, "developer-empty", 15))
 	message = _("The developer unit needs to be replaced.");
-		
+
       if (message)
       {
         strlcpy(buffer, _cupsLangString(lang, message), bufsize);
@@ -405,9 +390,9 @@ ppdLocalizeIPPReason(
 	    valptr ++;
 
 	    if (isdigit(*valptr & 255))
-	      *bufptr++ = ch | (*valptr - '0');
+	      *bufptr++ = (char)(ch | (*valptr - '0'));
 	    else
-	      *bufptr++ = ch | (tolower(*valptr) - 'a' + 10);
+	      *bufptr++ = (char)(ch | (tolower(*valptr) - 'a' + 10));
 	    valptr ++;
 	  }
 	  else if (*valptr == '+')
@@ -499,7 +484,7 @@ ppdLocalizeIPPReason(
  * text from the attribute value. If no localized text for the requested
  * name can be found, @code NULL@ is returned.
  *
- * @since CUPS 1.4/Mac OS X 10.6@
+ * @since CUPS 1.4/OS X 10.6@
  */
 
 const char *				/* O - Value or @code NULL@ if not found */
@@ -637,15 +622,15 @@ _ppdGetLanguages(ppd_file_t *ppd)	/* I - PPD file */
 /*
  * '_ppdHashName()' - Generate a hash value for a device or profile name.
  *
- * This function is primarily used on Mac OS X, but is generally accessible
+ * This function is primarily used on OS X, but is generally accessible
  * since cupstestppd needs to check for profile name collisions in PPD files...
  */
 
 unsigned				/* O - Hash value */
 _ppdHashName(const char *name)		/* I - Name to hash */
 {
-  int		mult;			/* Multiplier */
-  unsigned	hash = 0;		/* Hash value */
+  unsigned	mult,			/* Multiplier */
+		hash = 0;		/* Hash value */
 
 
   for (mult = 1; *name && mult <= 128; mult ++, name ++)
@@ -679,8 +664,23 @@ _ppdLocalizedAttr(ppd_file_t *ppd,	/* I - PPD file */
   snprintf(lkeyword, sizeof(lkeyword), "%s.%s", ll_CC, keyword);
   if ((attr = ppdFindAttr(ppd, lkeyword, spec)) == NULL)
   {
-    snprintf(lkeyword, sizeof(lkeyword), "%2.2s.%s", ll_CC, keyword);
-    attr = ppdFindAttr(ppd, lkeyword, spec);
+   /*
+    * <rdar://problem/22130168>
+    *
+    * Hong Kong locale needs special handling...  Sigh...
+    */
+
+    if (!strcmp(ll_CC, "zh_HK"))
+    {
+      snprintf(lkeyword, sizeof(lkeyword), "zh_TW.%s", keyword);
+      attr = ppdFindAttr(ppd, lkeyword, spec);
+    }
+
+    if (!attr)
+    {
+      snprintf(lkeyword, sizeof(lkeyword), "%2.2s.%s", ll_CC, keyword);
+      attr = ppdFindAttr(ppd, lkeyword, spec);
+    }
 
     if (!attr)
     {
@@ -727,8 +727,8 @@ _ppdLocalizedAttr(ppd_file_t *ppd,	/* I - PPD file */
  */
 
 static cups_lang_t *			/* O - Current language */
-ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
-          int  ll_CC_size)		/* I - Size of country-specific name */
+ppd_ll_CC(char   *ll_CC,		/* O - Country-specific locale name */
+          size_t ll_CC_size)		/* I - Size of country-specific name */
 {
   cups_lang_t	*lang;			/* Current language */
 
@@ -775,5 +775,5 @@ ppd_ll_CC(char *ll_CC,			/* O - Country-specific locale name */
 
 
 /*
- * End of "$Id: localize.c 10270 2012-02-13 17:13:21Z mike $".
+ * End of "$Id: localize.c 12834 2015-08-06 13:56:32Z msweet $".
  */
