@@ -1,18 +1,18 @@
 /*
- * "$Id: string-private.h 11021 2013-06-06 22:08:14Z msweet $"
+ * "$Id: string-private.h 12928 2015-10-23 21:31:58Z msweet $"
  *
- *   Private string definitions for CUPS.
+ * Private string definitions for CUPS.
  *
- *   Copyright 2007-2013 by Apple Inc.
- *   Copyright 1997-2006 by Easy Software Products.
+ * Copyright 2007-2015 by Apple Inc.
+ * Copyright 1997-2006 by Easy Software Products.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  *
- *   This file is subject to the Apple OS-Developed Software exception.
+ * This file is subject to the Apple OS-Developed Software exception.
  */
 
 #ifndef _CUPS_STRING_PRIVATE_H_
@@ -28,6 +28,7 @@
 #  include <ctype.h>
 #  include <errno.h>
 #  include <locale.h>
+#  include <time.h>
 
 #  include "config.h"
 
@@ -146,6 +147,7 @@ extern int _cups_toupper(int ch);
  * Prototypes...
  */
 
+extern ssize_t	_cups_safe_vsnprintf(char *, size_t, const char *, va_list);
 extern void	_cups_strcpy(char *dst, const char *src);
 
 #  ifndef HAVE_STRDUP
@@ -199,6 +201,13 @@ extern double	_cupsStrScand(const char *buf, char **bufptr,
 		              struct lconv *loc);
 
 
+/*
+ * Date function...
+ */
+
+extern char	*_cupsStrDate(char *buf, size_t bufsize, time_t timeval);
+
+
 /*
  * C++ magic...
  */
@@ -210,5 +219,5 @@ extern double	_cupsStrScand(const char *buf, char **bufptr,
 #endif /* !_CUPS_STRING_H_ */
 
 /*
- * End of "$Id: string-private.h 11021 2013-06-06 22:08:14Z msweet $".
+ * End of "$Id: string-private.h 12928 2015-10-23 21:31:58Z msweet $".
  */
